// c++ headers
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <iomanip.h>
// headers from ROOT
#include "TROOT.h"
#include "TFile.h"
#include "TProfile.h"
#include "TNtuple.h"
// our headers
#include "Lorenz.h"
#include "koralw.h"
#include "Koralw.h"
#include "partit1.h"
//
#include "rmain.h" //????
//  long NevTot = 1000000000; // total (limit) of events
//  long iEvent =0;           // consecutive event number in MC generation
//
void   RobolMaly(Koralw current_event, double xpar[], long npar[]);
void   read_param( long &NevTot, double xpar[], long npar[] );
void   JetDefine(Koralw *event);
void   BEbook(Koralw event, int kfP, int ntot, double wt, double wt2);
//
void   BoseEinstein(Koralw event, 
		    double lambda, double avewt, double lambda2, double avewt2,
		    int &nPiPlus, int &nPiMinus, int &nPiZero,
		    double &wtPlus, double &wtMinus, double &wtZero,
		    double &wtot,  double &wtot2);
double pair_Q2(double p1[4], double p2[4]);
double Q2pair(PartLund first, PartLund second);
void   parton_mass(Koralw event, double m[6] );
void   jet_mass(Koralw event, double m[6] );
//*****************************************************************************
//*****************************************************************************
void ROBOL(TString Mode, long &NevTot, long &iEvent)
{
  double xsect,ersect;
  static double xpar[101];
  static long   npar[101];

  if(  Mode == "Init" )
    {
      //#############KORALW initialisation############
      read_param( NevTot, xpar, npar );
      Koralw::initialize(xpar, npar);
      //##############################################
    }
  else if( Mode == "Prod" )
    {
      //KORALW event generation
      //*********************
      Koralw *current_event = new Koralw();
      //***************************************
      JetDefine( current_event );
      //***************************************
      // Test printouts
      if(iEvent < 10 ) 
	{
	  //current_event.print();  // print from class koralw
	  (*current_event).Print(); // big Print from class Koralw
	  lulist_(1);               // print from JETSET
	  cout << "njet = " << (*current_event).njet << endl;
	}
      //************************************
      RobolMaly( (*current_event), xpar, npar );
      //************************************
      delete current_event;
    }
  else if( Mode == "Fini" )
    {
      //**************  KORALW final printouts  **************
      Koralw::finalize( xpar, npar );
      xsect = xpar[20];
      ersect= xpar[21];
      cout << " KORALW: xsect  ="<<  xsect << "    ersect ="<< ersect << "\n" ;
      //******************************************************
    }
  //
};

//*****************************************************************************
//*****************************************************************************
void RobolMaly(Koralw current_event, double xpar[], long npar[])
{
  int kf;
// %%%%%%%%%%%%%%%%%%%%%%%%%
// BoseEinstein
     int nPiPlus; int nPiMinus; int nPiZero;
     double wtPlus; double wtMinus; double wtZero, wtBE, wtBE2;
     double lambda  = xpar[25];
     double avewt   = xpar[26];
     double lambda2 = xpar[27];
     double avewt2  = xpar[28];
     BoseEinstein(current_event,
		  lambda, avewt, lambda2, avewt2,
		  nPiPlus, nPiMinus, nPiZero,
		  wtPlus,  wtMinus,  wtZero,
		  wtBE, wtBE2);
     int ntot = nPiPlus+ nPiMinus+ nPiZero;
     kf= 211; BEbook(current_event,kf,ntot,wtBE,wtBE2);
     kf=-211; BEbook(current_event,kf,ntot,wtBE,wtBE2);
     //kf= 111; BEbook(current_event,kf,ntot,wtBE,wtBE2);

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Partons Partons Partons Partons Partons
     int i,j;
     hst_pene->Fill(current_event.ferm1[0]);
     hst_pene->Fill(current_event.ferm2[0]);
     double mWW[6];
     parton_mass(current_event,mWW);
     ntuple8->Fill(mWW[0],mWW[1],mWW[2],mWW[3],mWW[4],mWW[5],ntot,wtBE);
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Jets Jets Jets Jets Jets Jets Jets Jets Jets
     int nj = current_event.njet;
     for (j=0; j<nj ;j++) 
       hst_jene->Fill(current_event.jet[j].pmom[0]);

     jet_mass(current_event,mWW);
     jtuple6->Fill(mWW[0],mWW[1],nj,ntot,wtBE,wtBE2);
     jtuple6->Fill(mWW[2],mWW[3],nj,ntot,wtBE,wtBE2);
     jtuple6->Fill(mWW[4],mWW[5],nj,ntot,wtBE,wtBE2);

     if( nj == 4 ){
       // chlopologicznie bez wagi
       if(abs(mWW[0]-mWW[1])<10.00) MasW_avg->Fill(0.5*(mWW[0]+mWW[1]));
       if(abs(mWW[2]-mWW[3])<10.00) MasW_avg->Fill(0.5*(mWW[2]+mWW[3]));
       if(abs(mWW[4]-mWW[5])<10.00) MasW_avg->Fill(0.5*(mWW[4]+mWW[5]));
       // chlopologicznie z waga
       if(abs(mWW[0]-mWW[1])<10.00) MasW_avg2->Fill(0.5*(mWW[0]+mWW[1]),wtBE);
       if(abs(mWW[2]-mWW[3])<10.00) MasW_avg2->Fill(0.5*(mWW[2]+mWW[3]),wtBE);
       if(abs(mWW[4]-mWW[5])<10.00) MasW_avg2->Fill(0.5*(mWW[4]+mWW[5]),wtBE);
     };
};



/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
void parton_mass(Koralw event, double m[6] )
{
  VLorenz psum;
  psum = event.ferm1 + event.ferm2; m[0] = sqrt(psum*psum);
  psum = event.ferm3 + event.ferm4; m[1] = sqrt(psum*psum);
  psum = event.ferm1 + event.ferm3; m[2] = sqrt(psum*psum);
  psum = event.ferm2 + event.ferm4; m[3] = sqrt(psum*psum);
  psum = event.ferm1 + event.ferm4; m[4] = sqrt(psum*psum);
  psum = event.ferm2 + event.ferm3; m[5] = sqrt(psum*psum);
  //for(int i=0;i<6;i++) cout << m[i] << " "; cout << endl;
};
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void jet_mass(Koralw event, double m[6] )
{
  VLorenz psum;
  int nj = event.njet;
  for(int i=0; i<6; i++) m[i]=0;
  //[[[[[[[[[[[[[
  //cout << "==>" << event.jet[0].pmom[0] << " "; cout << endl;
  //cout << "..."; (*event.jet[0]).print();
  //cout << "..."; (*event.jet[1]).print();
  //cout << "..."; (*event.jet[2]).print();
  //cout << "..."; (*event.jet[3]).print();
  //]]]]]]]]]]]]]
  if(nj==4)
    {
      psum= event.jet[0].pmom +event.jet[1].pmom; m[0] =  sqrt(psum*psum);
      psum= event.jet[2].pmom +event.jet[3].pmom; m[1] =  sqrt(psum*psum);
      psum= event.jet[0].pmom +event.jet[2].pmom; m[2] =  sqrt(psum*psum);
      psum= event.jet[1].pmom +event.jet[3].pmom; m[3] =  sqrt(psum*psum);
      psum= event.jet[0].pmom +event.jet[3].pmom; m[4] =  sqrt(psum*psum);
      psum= event.jet[1].pmom +event.jet[2].pmom; m[5] =  sqrt(psum*psum);
    }
  //for(int i=0;i<6;i++) cout << m[i] << " "; cout << endl;
};

/*****************************************************************************/
//===========================================================================//
//                          Bose Einstein                                    //
//===========================================================================//
//***************************************************************************//
void BEbook(Koralw event, int kfP, int ntot, double wt, double wt2)
{
  int j,k,l;
  PartLund *pion[500];

  //cout<<"***********************************************"<<endl; 
  int nP=0;
  for (j=0; j< event.npart ;j++)
    if( event.part[j].kflavor ==  kfP ) 
      {
	pion[nP] = &event.part[j];
	nP++;
      };

  //cout<<"----------------------------------------------"<<endl; 
  //cout<<" nP= "<<nP<<endl;
  //cout<<" wt= "<<wt<<endl;

  //pairs (off-diagonal elements)
  for(j=0; j<nP;j++)
      for(k=0; k<j;k++)
	{
	  double Q2jk= Q2pair(*pion[j],*pion[k]);
	  hst_unQ2->Fill(Q2jk);
	  hst_wtQ2->Fill(Q2jk,wt);
	  hst_wt2Q2->Fill(Q2jk,wt2);
	  if(ctuple2_counter < ctuple2_max)
	    if(Q2jk<1.0) {ctuple2->Fill(Q2jk,ntot,wt); ctuple2_counter++;}
	  for(l=0; l<k;l++)
	    {
	      double Q2jl= Q2pair(*pion[j],*pion[l]);
	      double Q2kl= Q2pair(*pion[k],*pion[l]);
	      double Q3jkl = Q2jk+Q2jl+Q2kl;
	      hst_unQ3->Fill(Q3jkl);
	      hst_wtQ3->Fill(Q3jkl,wt);
	      hst_wt2Q3->Fill(Q3jkl,wt2);
	      if(ctuple3_counter < ctuple3_max)
		if(Q3jkl<1.0){ctuple3->Fill(Q3jkl,ntot,wt); ctuple3_counter++;}
	    }
	}

}
//==========================================================================//
//==========================================================================//
double Q2pair(PartLund first, PartLund second)
// calculates positive Q2 = -(q1-q2)^2 for a pair of particles
{
  VLorenz  Q;
  double   Q2;

  Q = first.pmom - second.pmom;
  Q2 = -(Q*Q);
  return Q2;
};
//==========================================================================//
//                    Correlation functions
//==========================================================================//
double CorFun1(double Q2, int mult, double Rf, double pp)
  // EXPONENTIAL UA1 fit
  {
      double R = Rf/0.2;                   // conversion to GeV^-1
      R = R*sqrt((mult*(mult-1))/2.0);     // UA1 fit, R increasing with n
      double Q2p = Q2*2/(mult*(mult-1));
      double Qp  = sqrt(Q2p);
      double x = Qp*R;
      double zexp= 0; 
      if( x < 100.0 ) zexp = exp(-x);
      return zexp;
  }
//=======================================================================
double CorFun2(double Q2, int mult, double Rf, double pp)
  // GAUSSIAN UA1 fit
  {
      double R = Rf/0.2;                   // conversion to GeV^-1
      double Q2p = Q2*2/(mult*(mult-1));
      double x = Q2p*(R*R);
      double zexp= 0; 
      if( x < 100.0 ) zexp = exp(-x);
      return zexp;
  }
//=======================================================================
void WTcluster(int mult, PartLund *first, double &wt)
//*************************************************************************//
// Operates on a SINGLE cluster *first which is still in form of a 
// LIST starting at the element *first,
// Calculates weight wt.
//*************************************************************************//
{ 
  int i,j,k,l,m,n;
  int ntot;
  float x;
  PartLund *clmat   = new PartLund[mult];
  float A[100][100];
  int link[100][100];
  int noli[100];
  float suli[100];

  // Copy etire cluster into matrix clmat
  PartLund *actual; j=0;
  for (actual = first; actual != NULL; actual = actual->previous)
    {
      clmat[j] = *actual;
      j++;
    };


  // real    matrix A containing the value of relative Q2
  for(j=0; j<mult;j++) A[j][j]=0.0;  // diagonal elements
  double Q2jk;
  double Q2 = 0;
  for(j=0; j<mult;j++)               // off-diagonal elements
      for(k=0; k<j;k++)
	{
	  double Q2jk= Q2pair( clmat[j],clmat[k]);
	  A[j][k]=Q2jk; A[k][j]=Q2jk;
	  Q2=Q2 + Q2jk;
	}

  wt=1;
  if(mult>1)
    {
      double pp;   // coherence parameter pp=p=lambda
      double R ;   // radius in fermi <===== !!!!!
      double zexp;
      int ifun=1;
      if( ifun == 2 )
	{
	  // exponential UA1 fit
	  pp=0.40; R=1.3; zexp =CorFun1(Q2, mult, R, pp);
	}
      else
	{
	  // gaussian    UA1 fit
	  pp=0.20; R=1.0; zexp =CorFun2(Q2, mult, R, pp);
	}
      // parameters of Kacper
      double pz=pp*zexp;
      double uz=1-pp;
      double wwjap;
    //**********************
      japww(mult,pz,uz,wwjap);
    //**********************
      wt=wwjap;
      if(wt>100.0)
	{
	  double Qp = sqrt(Q2*2/(mult*(mult-1)));
	  cout<<"mult= "<<mult<<" wt= "<<wt;
	  cout<<" pz="<<pz<<" pp="<<pp<<" Qp= "<<Qp ;
	  cout<<" zexp= "<<zexp<<endl;
	}
    }

  // IMPORTANT!!! detach dynamical storage!!!
  // IMPORTANT!!! detach dynamical storage!!!
  delete [] clmat;
};
//*********************************************************//
void BEchain2(Koralw event, int kfP, int &nP, double &wt)
//==========================================================//
// Forms chain of MANY CLUSTERS of particles type kfP
//==========================================================//
{
  int iPrint =0;  // Printing flag for debugs
  int j,k;
  double Qp,QpMin;
  PartLund *head, *actual;
  PartLund *first, *second;
  PartLund *clhead, *cltail, *next;
  PartLund *clust[500];
  int      nclust[500];
  int mulclus,np;
  int nptot=0;
  double Range=0.20; // maximum sqrt(Q2) range for a single pair to qualify

  nP=0; // default value 
  wt=1; // default value
  
  if(iPrint) cout << "||||||||||||||| BEchain2 Begin ||||||||||||||||" << endl;
  head=NULL;  
  for (j=0; j< event.npart ;j++)
    if( event.part[j].kflavor ==  kfP ) 
      {
	event.part[j].previous = head;
	head   = &event.part[j];
	if(iPrint) {(*head).pmom.print(); cout<<endl;}
	nP++;
      };

  if(iPrint) event.Print();
  if(iPrint) {cout << "ALL: "; (*head).ListPrint(); cout<<endl;}
  if(iPrint) cout << "==============================================" << endl;

  if(head == NULL) goto End;

  // I this part we form clusters of particles as LISTS which
  // begin at pointer *clust[i], i< mulclus
  // multiplicity of i-th cluster is stored in nclust[i]

  for(mulclus = 0; mulclus< 500; mulclus++)
    {
      np=1;
      cltail = head;          // last element in cluster segment
      for (actual = head; actual->previous != NULL; )
	{
	  if(iPrint) {(*actual).print(0); cout << " <-actual " << endl;}
	  next = actual->previous;
	  QpMin = 1e20;
	  for(first =head; ; first = first->previous)
	    {
	      Qp= sqrt(Q2pair(*first, *next));
	      if(Qp < QpMin) QpMin=Qp;
    if(iPrint){(*first).print(0);cout<<" <-first,QpMin="<<QpMin<<endl;}
	      if( first == cltail ) goto EE;
	    };
	EE:
	  if(QpMin < Range)
	    {
	      if(iPrint) {(*next).print(0);cout<<" <-detach "<< endl;}
	      actual->previous = actual->previous->previous; // detach
	      next->previous   = head;                       // attach
	      head= next;                                    // attach
	      np++;
	    }
	  else
	    {
	      actual= actual->previous;  // move further on
	    };
	};
      clhead = head;              // head of cluster
      head   = cltail->previous;  // new head of the remainder
      cltail->previous = NULL;    // grounding end of cluster
      clust[mulclus] = clhead;
      nclust[mulclus] = np;
      //
      int mlt2=np*(np-1)/2;
      hst_nclu->Fill(np+0.001,mlt2);     // cluster multiplicity
      //
      nptot = nptot+np;
     
      if(iPrint)
	{
	  cout<<"-----------> " << mulclus <<endl;
	  cout<<"np= "<<np << ": "; (*clhead).ListPrint(); cout<<endl;
	  (*head).ListPrint(); cout << endl;
	}

      if(head == NULL ) goto EEX;
    };
EEX:

  if(iPrint)
    {
      if( kfP ==  211) cout << "+++ "; 
      if( kfP == -211) cout << "--- "; 
      if( kfP ==  111) cout << "000 "; 
      cout << "|| nptot= "<<nptot << ":  ";
      cout << "mulclus= "<< mulclus << ":  ";
      for (j=0; j<mulclus; j++) cout << nclust[j] << " ";
      cout << endl;
    }

  double wtclu;
  for (j=0; j<mulclus; j++)
    if( nclust[j] > 1)
      {
	WTcluster(nclust[j], clust[j], wtclu);
	wt=wt*wtclu;
      }

End:
  if(iPrint) cout << "||||||||||||||| BEchain2 End ||||||||||||||||" << endl;
};  // END of BEchain2

//==========================================================================//
//==========================================================================//
void BoseEinstein(Koralw event, 
		  double lambda, double avewt, double lambda2, double avewt2,
		  int &nPiPlus, int &nPiMinus, int &nPiZero,
		  double &wtPlus, double &wtMinus, double &wtZero, 
		  double &wtot, double &wtot2)
{
  int kf_PiPlus  =  211;
  int kf_PiMinus = -211;
  int kf_PiZero  =  111;

  BEchain2(event, kf_PiPlus,  nPiPlus,  wtPlus);
  BEchain2(event, kf_PiMinus, nPiMinus, wtMinus);
  BEchain2(event, kf_PiZero,  nPiZero,  wtZero);


  int ntot= nPiPlus+nPiMinus+nPiZero;
  wtot2  = wtPlus*wtMinus*wtZero*exp(-ntot *log(lambda2))/avewt2;

  double Avewt  = exp(log(avewt)/3); // qubic root
  wtPlus  = wtPlus *exp(-nPiPlus *log(lambda))/Avewt;
  wtMinus = wtMinus*exp(-nPiMinus*log(lambda))/Avewt;
  wtZero  = wtZero *exp(-nPiZero *log(lambda))/Avewt;
  
  wtot = wtPlus*wtMinus*wtZero;

  if(wtot>1000.0)
    {
      cout << " n = " <<" "<< event.npart  << ",  ";
      cout << nPiPlus <<" "<< nPiMinus <<" "<< nPiZero  << "  ";
      cout << " wtPlus  = " << wtPlus  << "   ";
      cout << " wtMinus = " << wtMinus << "   ";
      cout << " wtZero  = " << wtZero  << "   ";
      cout << " wtot = "    << wtot << endl;
    }
};
//==========================================================================//
//==========================================================================//
void JetDefine(Koralw *event)
{
  //cout << "|||||||||||||||| User routine, jetology ||||||||||||||||" << endl;
  static int icont=0; icont++;
  int i,j,k,l;


  //==========================================================//
  //                     JETOLOGY                             //
  //==========================================================//

  long Njet;
  luclus_(Njet);

  if ( Njet < 0 ) {
    cout << "====== WARNING ====="<< endl;
    cout << "Njet= " << Njet << endl;
  };

  long mstu[200]; float paru[200]; long mstj[200]; float parj[200];
  ludat1_out_(mstu, paru, mstj, parj);

  (*event).njet = mstu[3-1]; // jet multiplicity

  long nl; long kl[4000][5]; float pl[4000][5]; float vl[4000][5];
  lujets_out_(nl, kl, pl, vl);

  for ( int kj=0; kj < (*event).njet  ; kj++ )
    {
      int j = (*event).npart +kj;
      (*event).jet[kj].lserial  = (*event).npart +kj+1;  // serial number
      (*event).jet[kj].kstatus  = kl[j][1-1];   // k(j,1)
      (*event).jet[kj].kflavor  = kl[j][2-1];   // k(j,2)
      (*event).jet[kj].kparent  = kl[j][3-1];   // k(j,3)

      (*event).jet[kj].pmass    = pl[j][5-1];   // p(j,5)
      for ( k=0; k < 4 ; k++ )
	{
	  l=k+1; if( l == 4 ) l=0;
	  (*event).jet[kj].pmom[l]  = pl[j][k];   // four-momentum
	}
      (*event).part[j]=(*event).jet[kj];
    };
  //==========================================================//
  //               end   JETOLOGY                             //
  //==========================================================//
}


/*****************************************************************************/
void read_param( long &NevTot, double xpar[], long npar[] )
{
  int j;
  char txtdum [200];
#define SkipLine infile2.getline(txtdum,100,'\n') //<-- skip to the end of line

  ifstream infile2;
  infile2.open("koralw.input"); 
  SkipLine;
  infile2 >> NevTot; SkipLine;
  for (j=0; j<30; j++)
    {
      infile2 >> npar[j+1];  SkipLine;
      //cout <<npar[j+1]<<"\n";
    }; 
  SkipLine;
  for (j=0; j<30; j++)
    {
      infile2>> xpar[j+1];   SkipLine;
      //cout <<xpar[j+1]<<"\n";
    };
  infile2.close();
}

