#ifndef ROBOL_H
#define ROBOL_H
///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                CLASS ROBOL                                                //
//    Entire processing of single MC event (Factory of factories)            //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
#include "TROOT.h"

#include "KorEvent.h"
#include "KoralwMaker.h"
#include "BEwtMaker.h"
#include "JetAnalyzer.h"

class ROBOL : public TNamed {
public:
  ///////////////////////////////
  //   Three basic Makers      //
  KoralwMaker   KoralW;              // KoralW MC generator maker
  BEwtMaker     BEfactory;           // BE weight BEwt maker
  JetAnalyzer   J4factory;           // Jet maker, defines 4 jets
  /////////////////////////////////
  //  Current event     (static)
  KorEvent      current_event;       // Current MC event from KORALW (static)
  /////////////////////////////////
  //  Control Histograms         //
  TH1F       *hst_BEwt;              // BE weight before rejection
  TH1F       *hst_BEwtAR;            // BE weight After  Rejection
  /////////////////////////////////
  // rejection control variables //
  long       KeyRej;                 // rejection key (=0 for rejection ON)
  double     WtMax;                  // maximum rejection weight for BE weight
public:
  ROBOL() {
    hst_BEwt  = new TH1F("hst_BEwt" ,  "BE weight before rejection",50,0,10);
    hst_BEwtAR= new TH1F("hst_BEwtAR" ,"BE weight after  rejection",50,0,10);
  }
  ~ROBOL() {;}
public:
// Methods
void ROBOL::Initialize(long &NevTot);
void ROBOL::Production(long &iEvent);
void ROBOL::Finalize();
////////////////////////////////////////////////////////////////////////////
  ClassDef(ROBOL,1) // Entire processing of single MC event (Factory of factories)
};
////////////////////////////////////////////////////////////////////////////
#endif
