#ifndef VLorentz_H
#define VLorentz_H
//////////////////////////////////////////////////////////////////////////////
//                     CLASS   VLorentz                                     //
//                                                                          //
//     Lorenz four-vector algebra with natural usage:                       //
//     (overloaded operators: +,-,*,/,[])                                   //
//     v=v1+v2 for adding vectors                                           //
//     v=v1-v2 for subtraction                                              //
//     v2=a*v1 left multiplyication by scalar a                             //
//     v2=v1*a left multiplyication by scalar a                             //
//     a = v1*v2 dotproduct of two vectors                                  //
//     theta = v1/v2 calculate angle (rads) between                         //
//             3-vector components of two 4-vectors                         //
//                                                                          //
//                                                                          //
//                end of description  VLorentz                              //
//////////////////////////////////////////////////////////////////////////////

#include "TObject.h"

class VLorentz : public TObject {
//class VLorentz {
public:
                double m_comp[4];       // 4-momentum, comp[0] is energy
public:
                VLorentz()  {;}
                VLorentz(double en, double px,  double py,  double pz );
///////////////////   virtual      ~VLorentz() { printf("VLorentz%x\n",this);}
   virtual      ~VLorentz() {;}
   void         print();
public:
                double & operator[](int index);
  ClassDef(VLorentz,1)   // Lorentz four-vector with linear algebra
};

//////////////////////////////////////////////////////////////////////////////
//                     Overloading operators                                //
//////////////////////////////////////////////////////////////////////////////
VLorentz operator+(VLorentz p1, VLorentz p2);
VLorentz operator-(VLorentz p1, VLorentz p2);
double   operator*(VLorentz p1, VLorentz p2);
VLorentz operator*(double x, VLorentz p2);
VLorentz operator*(VLorentz p1, double x);
double   operator/(VLorentz p1, VLorentz p2);
//////////////////////////////////////////////////////////////////////////////
#endif
