      PROGRAM  Main
*///////////////////////////////////////////////////////////////////
*//                                                               //
*//   To execute:  make KWdemoCC03                                //
*//                make KWdemoGRCall                              //
*//                                                               //
*//   Generally user is supposed to provide his/her own main      //
*//   program. Nevertheless we provide simple 2 examples of       //
*//   the main program. This is first of them.                    //
*//   Its role is: (1) useful template                            //
*//            (2) 1-st time x-check that program runs correctly. //
*//                                                               //
*//   The essential part of this program is loop in which series  //
*//   of KoralW events is generated. It also reads input from     //
*//   disk file, no histograming (see Bdemo for histogramming),   //
*//   most of output is from the generator itself.  At the end    //
*//   of the program, MC integrated x-section of KoralW is        //
*//   compared with semi-analytical  result from Korwan.          //
*//                                                               //
*//   Two data sets are provided for separate 2 runs:             //
*//   KWdemo.input.CC03                                           //
*//      CC03 "signal" process is simulated, (internal matrix     //
*//      element), ISR is on, with hadronization, tau deacys,     //
*//      no CKM mixing, weighted events.                          //
*//                                                               //
*//   KWdemo.input.GRCall                                         //
*//      all four-fermion matrix element of GRACE (external ME),  //
*//      hadronization, tau deacys, weighted events.              //
*//                                                               //
*///////////////////////////////////////////////////////////////////
      IMPLICIT NONE
      SAVE  
*
      REAL*8       xpar(10000)
      CHARACTER*80 DiskFile
      INTEGER      nout,i,j,k,NevTot, KeyPho, KeyAcc, Iev
      INTEGER      KeyZon, Key4f, mout
      REAL*8       Xtot3e, XErr3e, XsBorn, ErBorn, CMSene
      REAL*8       XsecMC, XErrMC
      INTEGER      iReset,iTalk
*
*///////////////////////////////////////////////////////////////
*//        Define output file                                 //
*///////////////////////////////////////////////////////////////
      nout=16                      ! Default in KoralW is nout=16
      OPEN(nout,file='KWdemo.output')
      CALL LuGive("MSTU(11)=16;")  ! Output unit for Jetset
*///////////////////////////////////////////////////////////////
*//        Read DEFAULT input data file for KoralW            //
*///////////////////////////////////////////////////////////////
      DiskFile = './data_DEFAULTS'
      iReset   = 1
      CALL KW_ReaDataX( DiskFile,     iReset, 10000, xpar)
*///////////////////////////////////////////////////////////////
*//        Read input data file for THIS test                 //
*///////////////////////////////////////////////////////////////
      DiskFile = './KWdemo.input'
      iReset   = 0
      CALL KW_ReaDataX(  DiskFile,    iReset, 10000, xpar)
*///////////////////////////////////////////////////////////////
*//        KoralW initialisation                              //
*///////////////////////////////////////////////////////////////
      CALL KW_Initialize(xpar)
*///////////////////////////////////////////////////////////////
*//        MAIL LOOP                                          //
*///////////////////////////////////////////////////////////////
*===============================================================
      NevTot = 10000
      DO Iev=1,NevTot
         IF (MOD(Iev,1000) .EQ. 1) WRITE(*,*)' Iev = ',Iev
         CALL KW_Make
         IF( Iev .LE. 2) CALL lulist(2)
      ENDDO         
*===============================================================
*///////////////////////////////////////////////////////////////
*//          final printouts of KoralW                        //
*///////////////////////////////////////////////////////////////
      CALL KW_Finalize
* Get final total MC x-section to be printed below
      CALL KW_GetXSecMC(XSecMC,XErrMC)
*//////////////////////////////////////////////////////////////////
*//         SemiAnalytical O(alf3)exp. form Korwan               //
*//  To compare SemiAnal results with MC make physics sense      //
*//  for KeyZon = 0 and  key4f = 0,                              //
*//  otherwise: Korwan provides useful info on crude x-section   //
*//////////////////////////////////////////////////////////////////
      KeyZon =xpar(1044)
      Key4f  =xpar(1042)
      CMSene=xpar(1)            ! input for Korwan 
      KeyPho=0
      KeyAcc=1
      CALL Korwan(CMSene**2,0d0,1d0,KeyPho,KeyAcc,XsBorn,ErBorn)
      KeyPho=303 
      KeyAcc=1
      CALL Korwan(CMSene**2,0d0,1d0,KeyPho,KeyAcc,Xtot3e,XErr3e)
*     Final printouts of demo
      DO i=1,2
         mout = 6
         IF( i.EQ.2 ) mout = nout
         WRITE(mout,'(A60)') '============ demo ============'
         WRITE(mout,'(1X,F17.8, 4H  +-, F11.8, 1X, A30)') 
     $        XsecMC,XErrMC, 'MC Best, XPAR, KoralW'
         IF( (KeyZon .EQ. 0) .AND. (Key4f .EQ. 0) ) THEN
            WRITE(mout,'(1X,F17.8, 4H  +-, F11.8, 1X, A30)') 
     $           XsBorn,ErBorn,'SemiAnal Born, Korwan'
            WRITE(mout,'(1X,F17.8, 4H  +-, F11.8, 1X, A30)') 
     $           Xtot3e,XErr3e,'SemiAnal O(alf3)exp.LL, Korwan'
         ENDIF
         WRITE(mout,'(A60)') '========== End demo =========='
      ENDDO
      END        

    



