      SUBROUTINE eexx_wt_cor(wtcrud,svar,amel,iflav,vvmin,wtcort)
*     ***********************************************************   
*===================================================================*
* NOTE: This part of KORALW is still under construction!            *
*       If you want to use it, please contact the authors first.    *
*===================================================================*
!-------------------------------------------------------------------!
! The t-channel radiation correction weight for eexx final states   !
! with hard high PT radiative photons.                              !
! INPUT: wtcrud - "crude" weight                                    !
!        svar  - CMS energy squared                                 !
!        amel  - electron mass                                      !
!        iflav - final state particles flavours (PDG)               !
!        vvmin   - soft photon cut-off in CMS (min: E_gamma/E_beam) !
! OUTPUT: wtcort - correction weight                                !
!-------------------------------------------------------------------!
! IMPORTANT:                                                        !
!  1. IT WORKS ONLY WHEN PHOTOS IS ON !!!                           ! 
!  2. EVERYTHING HAS TO BE COMPILED IN DOUBLE PRECISION !!!         ! 
!-------------------------------------------------------------------!
! Written by: Wieslaw Placzek                     CERN, July 1998   !
! Last update: 30.07.1998           by: W.P.                        !
!-------------------------------------------------------------------!     
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      COMMON / momset / qeff1(4),qeff2(4),sphum(4),sphot(100,4),nphot
      COMMON / momdec / q1(4),q2(4),p1(4),p2(4),p3(4),p4(4)
      COMMON / cms_eff_momdec /
     $      effb1(4),effb2(4),p1e(4),p2e(4),p3e(4),p4e(4)
      SAVE / momset /, / momdec /, / cms_eff_momdec /

      DIMENSION iflav(4)
      DIMENSION beam1(4),beam2(4)                               
      DIMENSION pflab(4,4),pfeff(4,4)
      DIMENSION pelab(2,4),pplab(2,4),peeff(2,4),ppeff(2,4)
      DIMENSION telab(2),tplab(2),teeff(2),tpeff(2)  
      DIMENSION pf1(4),pf2(4),pf3(4),pf4(4),phota(100,4)

      wtcort = 1
      IF (wtcrud.EQ.0d0) RETURN
*! e- PDG code
      ide = 11
*! Count e-/e+
      nel = 0
      DO i = 1,4
         IF (IABS(iflav(i)).EQ.ide) nel = nel + 1
      ENDDO
*! Do nothing if less than 2 electrons in FS
      IF (nel.LT.2) RETURN
*! Beams 4-momenta in LAB
      CALL set_eff_beams(svar,amel,beam1,beam2)
*! Final state fermion 4-momenta
cc*!... from internal COMMON
cc      DO k = 1,4
cc         pf1(k) = p1(k)
cc         pf2(k) = p2(k)
cc         pf3(k) = p3(k)
cc         pf4(k) = p4(k)
cc      ENDDO
*!... from HEPEVT
      CALL getp4m(iflav(1),pf1)
      CALL getp4m(iflav(2),pf2)
      CALL getp4m(iflav(3),pf3)
      CALL getp4m(iflav(4),pf4)
*! Photons 4-momenta with energy above Emin
*!... soft cut-off
      eps = 0.02d0
      Emin = beam1(4)*eps
cc*!... from internal COMMON
cc      nphota = 0
cc      DO i = 1,nphot
cc         IF (sphot(i,4).GT.Emin) THEN
cc           nphota = nphota + 1
cc           DO k = 1,4
cc             phota(nphota,k) = sphot(i,k)
cc           ENDDO
cc         ENDIF
cc      ENDDO
*! ... from HEPEVT
      CALL getpho(Emin,phota,nphota)
      DO k = 1,4
         pflab(1,k) = pf1(k)
         pflab(2,k) = pf2(k)
         pflab(3,k) = pf3(k)
         pflab(4,k) = pf4(k)
         pfeff(1,k) = p1e(k)
         pfeff(2,k) = p2e(k)
         pfeff(3,k) = p3e(k)
         pfeff(4,k) = p4e(k)
      ENDDO
*! Find all e- and e+ in FS
      nef = 0
      npf = 0
      DO i = 1,4
         IF (iflav(i).EQ.ide) THEN
            nef = nef + 1
            DO k = 1,4
               pelab(nef,k) = pflab(i,k)
               peeff(nef,k) = pfeff(i,k)
            ENDDO
         ELSEIF (iflav(i).EQ.-ide) THEN
            npf = npf + 1
            DO k = 1,4
               pplab(npf,k) = pflab(i,k)
               ppeff(npf,k) = pfeff(i,k)
            ENDDO
         ENDIF
      ENDDO
*! e- transfers (in LAB and "effective beams" frames)
      DO i = 1,nef
        telab(i)=(beam1(4) - pelab(i,4))**2 - (beam1(3) - pelab(i,3))**2
     &          -(beam1(2) - pelab(i,2))**2 - (beam1(1) - pelab(i,1))**2
        teeff(i)=(effb1(4) - peeff(i,4))**2 - (effb1(3) - peeff(i,3))**2
     &          -(effb1(2) - peeff(i,2))**2 - (effb1(1) - peeff(i,1))**2
      ENDDO
*! e+ transfers (in LAB and "effective beams" frames)
      DO i = 1,npf
        tplab(i)=(beam2(4) - pplab(i,4))**2 - (beam2(3) - pplab(i,3))**2
     &          -(beam2(2) - pplab(i,2))**2 - (beam2(1) - pplab(i,1))**2
        tpeff(i)=(effb2(4) - ppeff(i,4))**2 - (effb2(3) - ppeff(i,3))**2
     &          -(effb2(2) - ppeff(i,2))**2 - (effb2(1) - ppeff(i,1))**2
      ENDDO
*! Correction weight for t-channel photon radiaction
      CALL TraCor(beam1,beam2,pelab,pplab,phota,nphota,telab,tplab,
     &            nef,npf,amel,eps,wtrad)
*! Correction weight for hard high PT photon matrix element in t-channel
      CALL MetCor(telab,tplab,teeff,tpeff,nef,npf,wtmat)
*! Total correction weight 
      wtcort = wtrad*wtmat
      END

      SUBROUTINE TraCor(pein,ppin,pefin,ppfin,phot,nphot,te,tp,nef,npf,
     &                  amel,eps,wt)
*     *****************************************************************
!-------------------------------------------------------------------!
! The t-channel radiation correction weight for eexx final states.  !
! INPUT: pein  - incoming electron 4-momentum                       !
!        ppin  - incoming positron 4-momentum                       !
!        pefin - final state electrons 4-monenta                    !
!        ppfin - final state positrons 4-momenta                    !
!        phot  - radiative photons 4-momenta                        !
!        nhot  - number of radiative photons                        !
!        te,tp - electron, positron lines transfers, resp.          !
!        nef,npf - numbers of final state electrons/positrons       ! 
!        amel  - electron mass                                      !
!        eps   - soft photon cut-off in CMS (min: E_gamma/E_beam)   !
! OUTPUT: wt - correction weight                                    !
!-------------------------------------------------------------------!
! Written by: Wieslaw Placzek                     CERN, July 1998   !
! Last update: 11.07.1998           by: W.P.                        !
!-------------------------------------------------------------------!     
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      DIMENSION pein(4),ppin(4),pefin(2,4),ppfin(2,4),phot(100,4)
      DIMENSION te(2),tp(2)  
      DIMENSION pef(4),ppf(4)

*! Find e- and e+ lines with lowest transfers
      iemin = 1
      IF ( ABS(te(1)) .GT. ABS(te(nef)) ) iemin = nef
      ipmin = 1
      IF ( ABS(tp(1)) .GT. ABS(tp(npf)) ) ipmin = npf
      DO k = 1,4
         pef(k) = pefin(iemin,k)
         ppf(k) = ppfin(ipmin,k)
      ENDDO
*! YFS formfactor correction
      CALL FmfCor(pein,ppin,pef,ppf,amel,eps,wtfmf)
*! S-factor correction
      CALL SfaCor(pein,ppin,pef,ppf,phot,nphot,amel,wtsfa)
*! Total correction weight
      wt = wtfmf*wtsfa
      END

      SUBROUTINE FmfCor(p1,p2,q1,q2,amel,eps,wt)
*     ******************************************
!-------------------------------------------------------------------!
! Yennie-Frautschi-Suura form-factor correction for eexx channels   !
! in KORALW (to get right t-channel photon radiation behaviour).    !
! INPUT: p1,p2 - initial state e-,e+ 4-momenta                      !
!        q1,q2 - final   state e-,e+ 4-momenta                      !
!        amel  - electron mass                                      !
!        eps   - soft photon cut-off in CMS (min: E_gamma/E_beam)   !
! OUTPUT: wt - correction weight                                    !
!-------------------------------------------------------------------!
! Written by: Wieslaw Placzek                     CERN, July 1998   !
! Last update: 08.07.1998           by: W.P.                        !
!-------------------------------------------------------------------!     
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION p1(4),p2(4),q1(4),q2(4)

      Emin = p1(4)*eps
* s-channel: initial state
      Rsi = Yepair(1,p1,p2,Emin,amel) 
* s-channel: final state
      Rsf = Yepair(1,q1,q2,Emin,amel) 
* t-channel
      Rt1 = Yepair(0,p1,q1,Emin,amel)
      Rt2 = Yepair(0,p2,q2,Emin,amel)
* u-channel
      Ru1 = Yepair(0,p1,q2,Emin,amel)
      Ru2 = Yepair(0,p2,q1,Emin,amel)
* Form-factor
      Ytot = Rsi + Rsf + Rt1 + Rt2 - Ru1 - Ru2 
      Ysim = Rsi + Rsf
      fofaco = EXP(Ytot - Ysim)
* Compensation weight
      wt = fofaco 
      END

      FUNCTION Yepair(is,p1,p2,Emin,amel)
*     ***********************************
!-------------------------------------------------------------------!
! Yennie-Frautschi-Suura infrared form-factor (virtual + real soft) !
! in the CMS for any pair of electrons and/or positrons.            !
! INPUT: is = 0 for t and u-channels                                !
!           = 1 for s-channel                                       !  
!        p1,p2 - leptons four-momenta                               !
!        Emin  - minimum energy of hard photons in CMS [in GeV]     !
!        amel  - electron mass [in GeV]                             ! 
!-------------------------------------------------------------------!
! Written by: Wieslaw Placzek                Knoxville, Oct. 1995   !
! Last update: 08.07.1998           by: W.P.                        !
!-------------------------------------------------------------------!     
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER ( pi = 3.1415926535897932D0, alfinv = 137.03604D0 )
      PARAMETER ( alfpi = 1/alfinv/pi )
      REAL*8 p1(4),p2(4)
!
      E1 = p1(4)
      E2 = p2(4)
      QQ = 2*( p1(4)*p2(4) -p1(3)*p2(3) -p1(2)*p2(2) -p1(1)*p2(1) )
      Biglog = LOG(QQ/amel**2)
      eps2 = Emin**2/E1/E2
      om = E1 + E2
      de = E1 - E2
      Dt = SQRT(QQ + de**2)
      Root = (Biglog -1)*LOG(eps2) +0.5*Biglog -1
! At low angles Remn should be approximately equal to: 
!           -pi**2/6,  for s, u channels; 
!            0,        for t channel. 
      Remn = pi**2/3 -0.5*LOG(E2/E1)**2 
     &      -0.25*LOG( (Dt+de)**2/(4*E1*E2) )**2
     &      -0.25*LOG( (Dt-de)**2/(4*E1*E2) )**2 
     &      -DILOGY( (Dt+om)/(Dt+de) ) -DILOGY( (Dt+om)/(Dt-de) )
     &      -DILOGY( (Dt-om)/(Dt+de) ) -DILOGY( (Dt-om)/(Dt-de) )
      FormFa = alfpi*(Root + Remn) 
! Extra term for s-channel 
      IF (is.EQ.1) FormFa = FormFa + alfpi*pi**2/2
      Yepair = FormFa
      END

      SUBROUTINE SfaCor(p1,p2,q1,q2,phot,nphot,amel,wt)
*     *************************************************
!-------------------------------------------------------------------!
! Yennie-Frautschi-Suura S-factor correction for eexx channels      !
! in KORALW (to get right t-channel photon radiation behaviour).    !
! INPUT: p1,p2 - initial state e-,e+ 4-momenta                      !
!        q1,q2 - final   state e-,e+ 4-momenta                      !
!        phot  - photons 4-momenta                                  !
!        nphot - number of photons                                  !
!        amel  - electron mass                                      !
! OUTPUT: wt - correction weight                                    !
!-------------------------------------------------------------------!
! Written by: Wieslaw Placzek                     CERN, July 1998   !
! Last update: 03.07.1998           by: W.P.                        !
!-------------------------------------------------------------------!     
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      PARAMETER ( pi = 3.1415926535897932D0, alfinv = 137.03604D0 )
      PARAMETER ( alfpi = 1/alfinv/pi )
      DIMENSION p1(4),p2(4),q1(4),q2(4),phot(100,4)
!
      wt = 1
      if (nphot.EQ.0) RETURN
      ame = amel
      p1p2 = p1(4)*p2(4)-p1(3)*p2(3)-p1(2)*p2(2)-p1(1)*p2(1)
      p1q1 = p1(4)*q1(4)-p1(3)*q1(3)-p1(2)*q1(2)-p1(1)*q1(1)
      p1q2 = p1(4)*q2(4)-p1(3)*q2(3)-p1(2)*q2(2)-p1(1)*q2(1)
      p2q1 = p2(4)*q1(4)-p2(3)*q1(3)-p2(2)*q1(2)-p2(1)*q1(1)
      p2q2 = p2(4)*q2(4)-p2(3)*q2(3)-p2(2)*q2(2)-p2(1)*q2(1)
      q1q2 = q1(4)*q2(4)-q1(3)*q2(3)-q1(2)*q2(2)-q1(1)*q2(1)

      sfcor = 1
      DO i = 1,nphot
* Scalar products of photon 4-momentum with other 4-momenta
        p1k = p1(4)*phot(i,4) -p1(3)*phot(i,3)
     &       -p1(2)*phot(i,2) -p1(1)*phot(i,1)
        p2k = p2(4)*phot(i,4) -p2(3)*phot(i,3)
     &       -p2(2)*phot(i,2) -p2(1)*phot(i,1)
        q1k = q1(4)*phot(i,4) -q1(3)*phot(i,3)
     &       -q1(2)*phot(i,2) -q1(1)*phot(i,1)
        q2k = q2(4)*phot(i,4) -q2(3)*phot(i,3)
     &       -q2(2)*phot(i,2) -q2(1)*phot(i,1) 
* Soft factors S-tilde (factor alpha/4pi^2 omitted!)
*--- s-channel
*... initial state  
        sfsi = 2*p1p2/(p1k*p2k) - (ame/p1k)**2 - (ame/p2k)**2
*... final state    
        sfsf = 2*q1q2/(q1k*q2k) - (ame/q1k)**2 - (ame/q2k)**2
*--- t-channel
*... electron line 
        sfte = 2*p1q1/(p1k*q1k) - (ame/p1k)**2 - (ame/q1k)**2
*... positron line 
        sftp = 2*p2q2/(p2k*q2k) - (ame/p2k)**2 - (ame/q2k)**2
*--- u-channel
*... e- in, e+ out 
        sfue = 2*p1q2/(p1k*q2k) - (ame/p1k)**2 - (ame/q2k)**2
*... e+ in, e- out
        sfup = 2*p2q1/(p2k*q1k) - (ame/p2k)**2 - (ame/q1k)**2
*--- u-channel
* Compensation weight
        sftot = sfsi + sfsf + sfte + sftp - sfue - sfup
        sfsim = sfsi + sfsf
        sfcor = sfcor *sftot/sfsim
      ENDDO
      wt = sfcor
      END

      SUBROUTINE MetCor(telab,tplab,teeff,tpeff,nef,npf,wt)
*     *****************************************************   
!-------------------------------------------------------------------!
! 'Ad hoc' correction for hard photon matrix element                ! 
! in eexx channels.                                                 !
! INPUT: telab - electron line transfers in LAB                     !
!        tplab - positron line transfers in LAB                     !
!        teeff - electron line transfers in "effective beam" frame  !
!        tpeff - positron line transfers in "effective beam" frame  !
!        nef,npf - numbers of final state electrons/positrons       ! 
! OUTPUT: wt - correction weight                                    !
!-------------------------------------------------------------------!
! Written by: Wieslaw Placzek                     CERN, July 1998   !
! Last update: 11.07.1998           by: W.P.                        !
!-------------------------------------------------------------------!     
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      DIMENSION telab(2),tplab(2),teeff(2),tpeff(2)  

*! Find the lowest effective transfer
      tefmin = 1d10
      DO i = 1,nef
        teeffa = ABS(teeff(i))
        IF (teeffa.LT.tefmin) THEN
           tefmin = teeffa
           rtr = teeff(i)/telab(i)
        ENDIF
      ENDDO
      DO i = 1,npf
        tpeffa = ABS(tpeff(i))
        IF (tpeffa.LT.tefmin) THEN
           tefmin = tpeffa
           rtr = tpeff(i)/tplab(i)
        ENDIF
      ENDDO
*! Transfer correction weight (if lowest effective transfer 
*! below TefLow and transfer ratio below RtrLow)
      TefLow = 1d0
      RtrLow = 1d0
      IF (tefmin.LT.TefLow .AND. rtr.LT.RtrLow) THEN 
         wt = rtr
      ELSE
         wt = 1d0
      ENDIF
      END

      SUBROUTINE getp4m(ipc,p)
*     ************************
* Gets 4-momentum p of a particle with PDG code ipc
      DOUBLE PRECISION p(4)
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVT/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      SAVE  /HEPEVT/
!
      DO ihep = 3,NHEP
         IF (ipc.EQ.idhep(ihep)) THEN
            DO k = 1,4
               p(k) = PHEP(k,ihep)
            ENDDO
            GOTO 10
         ENDIF
      ENDDO
      WRITE(*,*)'>>> GETP4M: No such a particle in HEPEVT; ipc = ',ipc
      STOP
 10   CONTINUE
      END

      SUBROUTINE getpho(Emin,phot,npho)
*     *********************************
* Gets 4-momenta "phot" and number "npho" of all photons in /HEPEVT/
* with energy above Emin
      DOUBLE PRECISION Emin,phot(100,4)
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVT/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      SAVE  /HEPEVT/
!
      npho = 0
      DO ihep = 3,NHEP
         IF (IDHEP(ihep).EQ.22 .AND. PHEP(4,ihep).GT.Emin) THEN
            npho = npho + 1
            DO k = 1,4
               phot(npho,k) = PHEP(k,ihep)
            ENDDO
         ENDIF
      ENDDO
      END











